searchData={"items":[{"type":"module","title":"s2","doc":"Here lies s2, a wrapper around the Erlang's [`sets`](https://www.erlang.org/doc/apps/stdlib/sets.html)\nmodule which gives you the `{version, 2}` representation without having to worry about it. This library\nexists because I was using [elp](https://github.com/WhatsApp/erlang-language-platform) during\nAdvent of Code and it kept complaining about using `{version, 2}` for `sets:new/1` and `sets:from_list/1`.\nThe documentation says,\n\n> This new representation will become the default in future Erlang/OTP versions. \n\nbut it hasn't happened yet so either this library will be useful or it will push Erlang/OTP\nto make it the default. Either way, he API surface is more similar to Haskell's \n[`Data.Set`](https://hackage.haskell.org/package/containers-0.7/docs/Data-Set.html) module\nbecause of my own personal preferences. For reference, the `{version, 2}` representation is just\na map where all the values are `[]`.\n\nIf you have any suggestions or find an issue please submit an issue on [github](https://github.com/chiroptical/s2)","ref":"s2.html"},{"type":"function","title":"s2.cartesian_product/2","doc":"Given two sets, pair every element in the first set with every element in the second.\n\n```erlang\n1> Product = s2:cartesian_product(s2:from_list([1, 2]), s2:from_list([3, 4])).\n#{{1,3} => [],{1,4} => [],{2,3} => [],{2,4} => []}\n2> Product =:= s2:from_list([{1, 3}, {1, 4}, {2, 3}, {2, 4}]).\ntrue\n```","ref":"s2.html#cartesian_product/2"},{"type":"function","title":"s2.delete/2","doc":"Remove an element from the set. It won't fail if the element doesn't exist.\n\n```erlang\n1> Set = s2:from_list([1, 2, 3]).\n#{1 => [],2 => [],3 => []}\n2> s2:delete(1, Set).\n#{2 => [],3 => []}\n3> s2:delete(4, Set).\n#{1 => [],2 => [],3 => []}\n```","ref":"s2.html#delete/2"},{"type":"function","title":"s2.difference/2","doc":"Given two sets, return a new set with elements of the second\nremoved from the first.\n\n```erlang\n1> s2:difference(s2:from_list([1, 2, 3]), s2:empty()).\n#{1 => [],2 => [],3 => []}\n2> s2:difference(s2:from_list([1, 2, 3]), s2:from_list([1])).\n#{2 => [],3 => []}\n```","ref":"s2.html#difference/2"},{"type":"function","title":"s2.empty/0","doc":"Return the empty set.\n\n```erlang\n1> s2:empty().\n#{}\n```","ref":"s2.html#empty/0"},{"type":"function","title":"s2.filter/2","doc":"Keep elements in the set for which the predicate returns `true`.\n\n```erlang\n1> S = s2:from_list([1, 2, 3, 4]).\n#{1 => [],2 => [],3 => [],4 => []}\n2> s2:filter(fun (X) -> X >= 2 end, S).\n#{2 => [], 3 => [], 4 => []}  \n```","ref":"s2.html#filter/2"},{"type":"function","title":"s2.fold/3","doc":"Given a binary operation on elements of the set and an accumulator,\nan initial accumulator, and a set collect the result of applying\nthe binary operation to each element of the set starting from\nthe initial accumulator.\n\n```erlang\n1> S = s2:from_list([1, 2, 3, 4]).\n#{1 => [],2 => [],3 => [],4 => []}\n2> s2:fold(fun (Elem, Acc) -> Elem + Acc end, 0, S).\n10\n```","ref":"s2.html#fold/3"},{"type":"function","title":"s2.from_list/1","doc":"Construct a set from a list of elements.\n\n```erlang\n1> s2:from_list([1, 2, 3]).\n#{1 => [], 2 => [], 3 => []}\n```","ref":"s2.html#from_list/1"},{"type":"function","title":"s2.insert/2","doc":"Add an element to a set.\n\n```erlang\n1> s2:insert(1, s2:singleton(2)).\n#{1 => [],2 => []}\n```","ref":"s2.html#insert/2"},{"type":"function","title":"s2.intersection/2","doc":"Given two sets, take the set intersection of them.\n\n```erlang\n1> s2:intersection(s2:singleton(1), s2:singleton(1)).\n#{1 => []}\n2> s2:intersection(s2:singleton(1), s2:singleton(2)).\n#{}\n```","ref":"s2.html#intersection/2"},{"type":"function","title":"s2.intersections/1","doc":"Given a list of sets, take the set intersection of all sets.\n\n```erlang\n1> s2:intersections([s2:singleton(1), s2:singleton(1)]).\n#{1 => []}\n2> s2:intersections([s2:singleton(1), s2:singleton(2)]).\n#{}\n```","ref":"s2.html#intersections/1"},{"type":"function","title":"s2.is_disjoint/2","doc":"Given two sets, determine if their intersection is the empty set.\nReturns `true` if the intersection is the empty set and `false` otherwise.\n\n```erlang\n1> s2:is_disjoint(s2:singleton(1), s2:singleton(2)).\ntrue\n2> s2:is_disjoint(s2:from_list([1, 2]), s2:singleton(2)).\nfalse\n```","ref":"s2.html#is_disjoint/2"},{"type":"function","title":"s2.is_element/2","doc":"Given an element and a set, if the element is contained in the set return\n`true` otherwise return `false`.\n\n```erlang\n1> s2:is_element(1, s2:singleton(1)).\ntrue\n2> s2:is_element(1, s2:singleton(2)).\nfalse\n```","ref":"s2.html#is_element/2"},{"type":"function","title":"s2.is_empty/1","doc":"Given a set, if it is the empty set return `true` else return `false`.\n\n```erlang\n1> s2:is_empty(s2:empty()).\ntrue\n2> s2:is_empty(s2:singleton(1)).\nfalse\n```","ref":"s2.html#is_empty/1"},{"type":"function","title":"s2.is_proper_subset/2","doc":"Given two sets, if every element of the first is contained in the second\n**and** the sets are different return `true` otherwise return `false`.\n\n```erlang\n1> s2:is_proper_subset(s2:singleton(1), s2:singleton(1)).\ntrue\n2> s2:is_proper_subset(s2:from_list([1, 2]), s2:from_list([1, 2])).\nfalse\n3> s2:is_proper_subset(s2:from_list([1, 2]), s2:from_list([1, 2, 3])).\ntrue\n```","ref":"s2.html#is_proper_subset/2"},{"type":"function","title":"s2.is_subset/2","doc":"Given two sets, if every element of the first is contained in the second\nreturn `true` otherwise return `false`.\n\n```erlang\n1> s2:is_subset(s2:singleton(1), s2:singleton(1)).\ntrue\n2> s2:is_subset(s2:from_list([1, 2]), s2:from_list([1, 2])).\ntrue\n3> s2:is_subset(s2:from_list([1, 2]), s2:from_list([1, 2, 3])).\ntrue\n```","ref":"s2.html#is_subset/2"},{"type":"function","title":"s2.map/2","doc":"Given a unary function and a set, apply the unary function to each element\nof the set to form a new set. The input and output set may not be the same\nsize.\n\n```erlang\n1> s2:map(fun (X) -> X + 1 end, s2:from_list([1, 2, 3])).\n#{2 => [],3 => [],4 => []}\n2> s2:map(fun (_X) -> 1 end, s2:from_list([1, 2, 3])).\n#{1 => []}\n```","ref":"s2.html#map/2"},{"type":"function","title":"s2.partition/2","doc":"partition(Pred, Set) -> {Keep, Discard}\n\nSimilar to `filter/2` but it returns a `Tuple` of two elements. The first\ncontains elements where the predicate evaluates to `true`. The second contains\nelements where the predicate evaluates to `false`.","ref":"s2.html#partition/2"},{"type":"function","title":"s2.singleton/1","doc":"Construct a set with a single element.\n\n```erlang\n1> s2:singleton(1).\n#{1 => []}\n```","ref":"s2.html#singleton/1"},{"type":"function","title":"s2.size/1","doc":"Given a set, return the number of elements in the set.\n\n```erlang\n1> s2:size(s2:empty()).\n0\n2> s2:size(s2:from_list([1, 2])).\n2\n```","ref":"s2.html#size/1"},{"type":"function","title":"s2.to_list/1","doc":"Given a set, return a list with all elements from the set.\n\n```erlang\n1> s2:to_list(s2:empty()).\n[]\n2> s2:to_list(s2:singleton(1)).\n[1]\n3> s2:to_list(s2:from_list([1, 2, 3])).\n[1,2,3]\n```","ref":"s2.html#to_list/1"},{"type":"function","title":"s2.union/2","doc":"Given two sets, return a new set with every element from both sets.\n\n```erlang\n1> s2:union(s2:singleton(1), s2:singleton(2)).\n#{1 => [],2 => []}\n```","ref":"s2.html#union/2"},{"type":"function","title":"s2.unions/1","doc":"Given a list of sets, return a new set with every element from\nevery set.\n\n```erlang\n1> s2:unions([s2:singleton(1), s2:singleton(2), s2:singleton(3)]).\n#{1 => [],2 => [],3 => []}\n```","ref":"s2.html#unions/1"},{"type":"opaque","title":"s2.set/1","doc":"As returned by `empty/0`.","ref":"s2.html#t:set/1"},{"type":"extras","title":"s2","doc":"s2\n=====\n\ns2 is very basic wrapper around Erlang's `sets` module. The API surface is very\nsimilar to Haskell's `Data.Set` library because that appeals to me. What does s2\ncode look like?\n\n```erlang\n1> X = s2:from_list([1, 2, 3]).\n#{1 => [], 2 => [], 3 => []}\n2> Y = s2:empty().\n#{}\n3> s2:union(X, Y).\n#{1 => [], 2 => [], 3 => []}\n```\n\nThe reason I decided to make this library was because the Erlang Language\nPlatform LSP kept bothering me about using `{version, 2}` when constructing\nsets in my Advent of Code solutions. I am lazy and I don't want to worry about\nremembering this. Additionally, day-to-day I write Haskell so the API naming of\n`Data.Set` is very easy for me to remember. If Erlang's maintainers switched to\nusing the `{version, 2}` in the next release of Erlang this library would still\nbe useful for me.\n\nPlease feel free to file an issue if you think other functions would be useful\nfor Advent of Code problems.\n\nContributing\n=====\n\nIf you are adding any new functionality please add a unit test, ensure `rebar3\neunit` passes and please run `rebar3 fmt -w` before submitting your pull\nrequest. In theory, you should be able to use any of Erlang 25, 26, or 27 to\ndevelop this library. I personally use `nix` to manage all of my development\nenvironments, including this repository.\n\nTo build the documentation, you'll need `ex_doc` (provided in nix shell). Run\n`rebar3 ex_doc` to ensure everything works.","ref":"readme.html"},{"type":"extras","title":"Apache License","doc":"# Apache License\nVersion 2.0, January 2004\n\nhttp://www.apache.org/licenses/\n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION","ref":"license.html"},{"type":"extras","title":"1. Definitions. - Apache License","doc":"\"License\" shall mean the terms and conditions for use, reproduction, and\ndistribution as defined by Sections 1 through 9 of this document.\n\n\"Licensor\" shall mean the copyright owner or entity authorized by the copyright\nowner that is granting the License.\n\n\"Legal Entity\" shall mean the union of the acting entity and all other entities\nthat control, are controlled by, or are under common control with that entity.\nFor the purposes of this definition, \"control\" means (i) the power, direct or\nindirect, to cause the direction or management of such entity, whether by\ncontract or otherwise, or (ii) ownership of fifty percent (50%) or more of the\noutstanding shares, or (iii) beneficial ownership of such entity.\n\n\"You\" (or \"Your\") shall mean an individual or Legal Entity exercising\npermissions granted by this License.\n\n\"Source\" form shall mean the preferred form for making modifications, including\nbut not limited to software source code, documentation source, and configuration\nfiles.\n\n\"Object\" form shall mean any form resulting from mechanical transformation or\ntranslation of a Source form, including but not limited to compiled object code,\ngenerated documentation, and conversions to other media types.\n\n\"Work\" shall mean the work of authorship, whether in Source or Object form, made\navailable under the License, as indicated by a copyright notice that is included\nin or attached to the work (an example is provided in the Appendix below).\n\n\"Derivative Works\" shall mean any work, whether in Source or Object form, that\nis based on (or derived from) the Work and for which the editorial revisions,\nannotations, elaborations, or other modifications represent, as a whole, an\noriginal work of authorship. For the purposes of this License, Derivative Works\nshall not include works that remain separable from, or merely link (or bind by\nname) to the interfaces of, the Work and Derivative Works thereof.\n\n\"Contribution\" shall mean any work of authorship, including the original version\nof the Work and any modifications or additions to that Work or Derivative Works\nthereof, that is intentionally submitted to Licensor for inclusion in the Work\nby the copyright owner or by an individual or Legal Entity authorized to submit\non behalf of the copyright owner. For the purposes of this definition,\n\"submitted\" means any form of electronic, verbal, or written communication sent\nto the Licensor or its representatives, including but not limited to\ncommunication on electronic mailing lists, source code control systems, and\nissue tracking systems that are managed by, or on behalf of, the Licensor for\nthe purpose of discussing and improving the Work, but excluding communication\nthat is conspicuously marked or otherwise designated in writing by the copyright\nowner as \"Not a Contribution.\"\n\n\"Contributor\" shall mean Licensor and any individual or Legal Entity on behalf\nof whom a Contribution has been received by Licensor and subsequently\nincorporated within the Work.","ref":"license.html#1-definitions"},{"type":"extras","title":"2. Grant of Copyright License. - Apache License","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable copyright license to reproduce, prepare Derivative Works of,\npublicly display, publicly perform, sublicense, and distribute the Work and such\nDerivative Works in Source or Object form.","ref":"license.html#2-grant-of-copyright-license"},{"type":"extras","title":"3. Grant of Patent License. - Apache License","doc":"Subject to the terms and conditions of this License, each Contributor hereby\ngrants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free,\nirrevocable (except as stated in this section) patent license to make, have\nmade, use, offer to sell, sell, import, and otherwise transfer the Work, where\nsuch license applies only to those patent claims licensable by such Contributor\nthat are necessarily infringed by their Contribution(s) alone or by combination\nof their Contribution(s) with the Work to which such Contribution(s) was\nsubmitted. If You institute patent litigation against any entity (including a\ncross-claim or counterclaim in a lawsuit) alleging that the Work or a\nContribution incorporated within the Work constitutes direct or contributory\npatent infringement, then any patent licenses granted to You under this License\nfor that Work shall terminate as of the date such litigation is filed.","ref":"license.html#3-grant-of-patent-license"},{"type":"extras","title":"4. Redistribution. - Apache License","doc":"You may reproduce and distribute copies of the Work or Derivative Works thereof\nin any medium, with or without modifications, and in Source or Object form,\nprovided that You meet the following conditions:\n\n1. You must give any other recipients of the Work or Derivative Works a copy of\n   this License; and\n\n2. You must cause any modified files to carry prominent notices stating that\n   You changed the files; and\n\n3. You must retain, in the Source form of any Derivative Works that You\n   distribute, all copyright, patent, trademark, and attribution notices from\n   the Source form of the Work, excluding those notices that do not pertain to\n   any part of the Derivative Works; and\n\n4. If the Work includes a \"NOTICE\" text file as part of its distribution, then\n   any Derivative Works that You distribute must include a readable copy of the\n   attribution notices contained within such NOTICE file, excluding those\n   notices that do not pertain to any part of the Derivative Works, in at least\n   one of the following places: within a NOTICE text file distributed as part\n   of the Derivative Works; within the Source form or documentation, if\n   provided along with the Derivative Works; or, within a display generated by\n   the Derivative Works, if and wherever such third-party notices normally\n   appear. The contents of the NOTICE file are for informational purposes only\n   and do not modify the License. You may add Your own attribution notices\n   within Derivative Works that You distribute, alongside or as an addendum to\n   the NOTICE text from the Work, provided that such additional attribution\n   notices cannot be construed as modifying the License.\n\nYou may add Your own copyright statement to Your modifications and may provide\nadditional or different license terms and conditions for use, reproduction, or\ndistribution of Your modifications, or for any such Derivative Works as a whole,\nprovided Your use, reproduction, and distribution of the Work otherwise complies\nwith the conditions stated in this License.","ref":"license.html#4-redistribution"},{"type":"extras","title":"5. Submission of Contributions. - Apache License","doc":"Unless You explicitly state otherwise, any Contribution intentionally submitted\nfor inclusion in the Work by You to the Licensor shall be under the terms and\nconditions of this License, without any additional terms or conditions.\nNotwithstanding the above, nothing herein shall supersede or modify the terms of\nany separate license agreement you may have executed with Licensor regarding\nsuch Contributions.","ref":"license.html#5-submission-of-contributions"},{"type":"extras","title":"6. Trademarks. - Apache License","doc":"This License does not grant permission to use the trade names, trademarks,\nservice marks, or product names of the Licensor, except as required for\nreasonable and customary use in describing the origin of the Work and\nreproducing the content of the NOTICE file.","ref":"license.html#6-trademarks"},{"type":"extras","title":"7. Disclaimer of Warranty. - Apache License","doc":"Unless required by applicable law or agreed to in writing, Licensor provides the\nWork (and each Contributor provides its Contributions) on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied,\nincluding, without limitation, any warranties or conditions of TITLE, NON-\nINFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are\nsolely responsible for determining the appropriateness of using or\nredistributing the Work and assume any risks associated with Your exercise of\npermissions under this License.","ref":"license.html#7-disclaimer-of-warranty"},{"type":"extras","title":"8. Limitation of Liability. - Apache License","doc":"In no event and under no legal theory, whether in tort (including negligence),\ncontract, or otherwise, unless required by applicable law (such as deliberate\nand grossly negligent acts) or agreed to in writing, shall any Contributor be\nliable to You for damages, including any direct, indirect, special, incidental,\nor consequential damages of any character arising as a result of this License or\nout of the use or inability to use the Work (including but not limited to\ndamages for loss of goodwill, work stoppage, computer failure or malfunction, or\nany and all other commercial damages or losses), even if such Contributor has\nbeen advised of the possibility of such damages.","ref":"license.html#8-limitation-of-liability"},{"type":"extras","title":"9. Accepting Warranty or Additional Liability. - Apache License","doc":"While redistributing the Work or Derivative Works thereof, You may choose to\noffer, and charge a fee for, acceptance of support, warranty, indemnity, or\nother liability obligations and/or rights consistent with this License. However,\nin accepting such obligations, You may act only on Your own behalf and on Your\nsole responsibility, not on behalf of any other Contributor, and only if You\nagree to indemnify, defend, and hold each Contributor harmless for any liability\nincurred by, or claims asserted against, such Contributor by reason of your\naccepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS\n\nCopyright 2024, chiroptical  .\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\nhttp://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"license.html#9-accepting-warranty-or-additional-liability"}],"proglang":"erlang","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}